@@ -34,12 +34,16 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.PropertyPermission;
 
 import org.apache.felix.framework.util.SecureAction;
 import org.osgi.framework.AdminPermission;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
+import org.osgi.framework.CapabilityPermission;
 import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.PackagePermission;
+import org.osgi.framework.ServicePermission;
 import org.osgi.framework.ServiceReference;
 import org.osgi.service.packageadmin.ExportedPackage;
 import org.osgi.service.packageadmin.PackageAdmin;
@@ -107,12 +111,15 @@ public final class Permissions
     {
         return new PermissionInfo[] {
             IMPLICIT[0],
-            new PermissionInfo(AdminPermission.class.getName(), "(id="
-                + bundle.getBundleId() + ")", AdminPermission.METADATA),
-            new PermissionInfo(AdminPermission.class.getName(), "(id="
-                + bundle.getBundleId() + ")", AdminPermission.RESOURCE),
-            new PermissionInfo(AdminPermission.class.getName(), "(id="
-                + bundle.getBundleId() + ")", AdminPermission.CONTEXT) };
+            new PermissionInfo(PropertyPermission.class.getName(), "org.osgi.framework.*", "read"),
+            new PermissionInfo(
+                AdminPermission.class.getName(),
+                "(id=" + bundle.getBundleId() + ")",
+                AdminPermission.CLASS + "," + AdminPermission.METADATA + "," + AdminPermission.RESOURCE + "," + AdminPermission.CONTEXT),
+            new PermissionInfo(CapabilityPermission.class.getName(), "(|(capability.namespace=osgi.ee)(capability.namespace=osgi.native))", CapabilityPermission.REQUIRE),
+            new PermissionInfo(PackagePermission.class.getName(),"(package.name=java.*)",PackagePermission.IMPORT),
+            new PermissionInfo(ServicePermission.class.getName(),"org.osgi.service.condition.Condition", ServicePermission.GET)
+        };
     }
 
     public Permissions getPermissions(PermissionInfo[] permissionInfos)
@@ -336,48 +343,7 @@ public final class Permissions
                     Permission source = createPermission(new PermissionInfo(
                         FilePermission.class.getName(), name,
                         m_permissionInfos[i].getActions()), targetClass);
-                    postfix = "";
-                    name = target.getName();
-                    if (!"<<ALL FILES>>".equals(name))
-                    {
-                        if (name.endsWith("*") || name.endsWith("-"))
-                        {
-                            postfix = name.substring(name.length() - 1);
-                            name = name.substring(0, name.length() - 1);
-                        }
-                        if (!(new File(name)).isAbsolute())
-                        {
-                            BundleContext context = (BundleContext) AccessController
-                                .doPrivileged(new PrivilegedAction()
-                                {
-                                    public Object run()
-                                    {
-                                        return bundle.getBundleContext();
-                                    }
-                                });
-                            if (context == null)
-                            {
-                                break;
-                            }
-                            name = m_action.getAbsolutePath(new File(context
-                                .getDataFile(""), name));
-                        }
-                        if (postfix.length() > 0)
-                        {
-                            if ((name.length() > 0) && !name.endsWith("/"))
-                            {
-                                name += "/" + postfix;
-                            }
-                            else
-                            {
-                                name += postfix;
-                            }
-                        }
-                    }
-                    Permission realTarget = createPermission(
-                        new PermissionInfo(FilePermission.class.getName(),
-                            name, target.getActions()), targetClass);
-                    if (source.implies(realTarget))
+                    if (source.implies(target))
                     {
                         return true;
                     }
@@ -643,9 +609,19 @@ public final class Permissions
         // target + "\n\n");
         try
         {
-            return (Permission) m_action.getConstructor(target,
-                new Class[] { String.class, String.class }).newInstance(
-                new Object[] { name, action });
+            try
+            {
+                return (Permission) m_action.getConstructor(target,
+                    new Class[] { String.class, String.class }).newInstance(
+                    new Object[] { name, action });
+            }
+            // Fall-back to action-less constructor
+            catch (NoSuchMethodException ex)
+            {
+                return (Permission) m_action.getConstructor(target,
+                    new Class[] { String.class }).newInstance(
+                    new Object[] { name });
+            }
         }
         catch (Exception ex)
         {
